// This file is part of PolyMPC, a lightweight C++ template library
// for real-time nonlinear optimization and optimal control.
//
// Copyright (C) 2020 Listov Petr <petr.listov@epfl.ch>
//
// This Source Code Form is subject to the terms of the Mozilla
// Public License v. 2.0. If a copy of the MPL was not distributed
// with this file, You can obtain one at http://mozilla.org/MPL/2.0/.

#include "mobile_robot.hpp"
#include "nmpc.hpp"

using namespace casadi;

int main(void)
{
    using nmpc_controller = polympc::nmpc<MobileRobot, 3, 2, 2, 4>;

    DMDict options;
    options["mpc.scaling"] = 0;
    options["mpc.scale_x"] = DM::diag(DM(std::vector<double>{1,1,1}));
    options["mpc.scale_u"] = DM::diag(DM(std::vector<double>{1,1}));

    DM Q  = casadi::SX::diag(casadi::SX(std::vector<double>{10,150,50}));
    DM R  = casadi::SX::diag(casadi::SX(std::vector<double>{1,1}));
    DM P  = 1e1 * Q;

    options["mpc.Q"] = Q;
    options["mpc.R"] = R;
    options["mpc.P"] = P;

    DM target  = DM(std::vector<double>{0, 0, 0});
    double tf = 2.0;
    nmpc_controller robot_controller(target, tf, options);

    /** set state and control constraints */
    DM lbu = DM::vertcat({-1, -0.6});
    DM ubu = DM::vertcat({ 1,  0.6});
    robot_controller.setLBU(lbu);
    robot_controller.setUBU(ubu);

    /** set state/virtual state constraints */
    DM lbx = DM::vertcat({-DM::inf(), -DM::inf(), -DM::inf()});
    DM ubx = DM::vertcat({ DM::inf(),  DM::inf(),  DM::inf()});
    robot_controller.setLBX(lbx);
    robot_controller.setUBX(ubx);

    DM state = DM(std::vector<double>{ -1, -1, 0});
    robot_controller.computeControl(state);

    return 0;
}


